#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
#include "INLINE.h"

#include <X11/Xlib.h>
#include <X11/X.h>
#include <X11/keysym.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <shadow.h>
#include <crypt.h>

SV *open_connection(){
    Display *display;
    SV *d_sv;
    SV *d_sv_ref;
    display = XOpenDisplay(NULL);
    if (display == NULL){
        croak("Could not open X display connection.");
    }
    d_sv = newSViv((IV)display);
    d_sv_ref = newRV_noinc(d_sv);
    return d_sv_ref;
}

void close_connection(SV* display){
    XCloseDisplay((Display *)SvIV(SvRV(display)));
}

void destroy_window(SV *display, int window){
    XDestroyWindow((Display *)SvIV(SvRV(display)), window);
}

int screen_count(SV *display){
    return XScreenCount((Display *)SvIV(SvRV(display)));
}

int close_window(SV *display, int window){
    XDestroyWindow((Display *)SvIV(SvRV(display)), window);
}

int open_lock(SV *disp_ref, int screen){
    Display *display;
    int window;
    int root;
    int status;
    XColor black, d;
    XSetWindowAttributes attr;
    XEvent event;

    display = (Display *)SvIV(SvRV(disp_ref));
    screen = XDefaultScreen(display);
    root = XRootWindow(display,screen);
    window = XCreateSimpleWindow(display, root, 0, 0, DisplayWidth(display,screen), DisplayHeight(display,screen), 1, BlackPixel(display, screen), BlackPixel(display, screen));
    attr.override_redirect = 1;
    XChangeWindowAttributes(display, window, CWOverrideRedirect, &attr);

    status = XMapRaised(display, window);
    XRaiseWindow(display, window);

    XSync(display,0);

    XSelectInput(display, root, SubstructureNotifyMask);
    return window;
} 

int grab_keyboard(SV *disp_ref, int screen){
    Display *display;
    int root;
    int kgrab;
    int mgrab; 

    display = (Display *)SvIV(SvRV(disp_ref));
    root = XRootWindow(display, screen);
    kgrab = XGrabKeyboard(display, root, True, GrabModeAsync, GrabModeAsync, CurrentTime);
    mgrab = XGrabPointer(display, root, False, ButtonPress | PointerMotionMask, GrabModeAsync, GrabModeAsync, None, None, CurrentTime);
    return (kgrab > 0 && mgrab > 0);
}

void ungrab_keyboard(SV* disp_ref){
    Display *display;
    display = (Display *)SvIV(SvRV(disp_ref));
    XUngrabKeyboard(display, CurrentTime);
    XUngrabPointer(display, CurrentTime);
}

int read_event(SV *d){
    Display *display;
    char buffer[64];
    XEvent event;
    display = (Display *)SvIV(SvRV(d));
    KeySym sym;
    XNextEvent(display, &event);
    int num;

    if (event.type == KeyPress){
         num = XLookupString(&event.xkey, buffer, sizeof buffer, &sym, 0);
     if (num == 1) {
        if (sym == XK_Return){
            return 10;
        }else {
            return buffer[0];
        }
     }

    }
    return -1;
}


int check_unix_password(char *checkpw){
    struct spwd *entry;
    char *pw;
    entry = getspnam(getenv("USER"));
    pw = entry->sp_pwdp;
    return strcmp(pw, crypt(checkpw, pw));
}

=head1 NAME

App::PerlXLock - A simple X locking utility.

=head1 SYNOPSIS

    #lock without password
    plock 

    #When using perlbrew, the following zsh command runs lock with password read from /etc/shadow.
    #Add NOPASSWD to sudoers accordingly. 
    sudo $(which perl) $(which plock)

    #lock with password given in an environmental variable
    PLOCK_PASSWORD="password" plock

=head1 DESCRIPTION

A X11 locking utility. plock turns the screen black and waits for a key press. If /etc/shadow
is not accessible and PLOCK_PASSWORD is not set, plock terminates after pressing return. If /etc/shadow
is found to be readable, then it waits until the user has typed his password and pressed return. 

In case of mistyping the password, simply retype as plock detects once the correct password
appears in the buffer.

The module uses libX11, which must be installed prior to installing this module. 

=head1 CAVEATS

At the moment, recognizes only a shadow password file or the environmental variable.

=head1 LICENSE

This software is licensed under the same terms as Perl itself.

=cut

MODULE = App::PerlXLock::Inline  PACKAGE = App::PerlXLock  

PROTOTYPES: DISABLE


SV *
open_connection ()

void
close_connection (display)
	SV *	display
        PREINIT:
        I32* temp;
        PPCODE:
        temp = PL_markstack_ptr++;
        close_connection(display);
        if (PL_markstack_ptr != temp) {
          /* truly void, because dXSARGS not invoked */
          PL_markstack_ptr = temp;
          XSRETURN_EMPTY; /* return empty stack */
        }
        /* must have used dXSARGS; list context implied */
        return; /* assume stack size is correct */

void
destroy_window (display, window)
	SV *	display
	int	window
        PREINIT:
        I32* temp;
        PPCODE:
        temp = PL_markstack_ptr++;
        destroy_window(display, window);
        if (PL_markstack_ptr != temp) {
          /* truly void, because dXSARGS not invoked */
          PL_markstack_ptr = temp;
          XSRETURN_EMPTY; /* return empty stack */
        }
        /* must have used dXSARGS; list context implied */
        return; /* assume stack size is correct */

int
screen_count (display)
	SV *	display

int
close_window (display, window)
	SV *	display
	int	window

int
open_lock (disp_ref, screen)
	SV *	disp_ref
	int	screen

int
grab_keyboard (disp_ref, screen)
	SV *	disp_ref
	int	screen

void
ungrab_keyboard (disp_ref)
	SV *	disp_ref
        PREINIT:
        I32* temp;
        PPCODE:
        temp = PL_markstack_ptr++;
        ungrab_keyboard(disp_ref);
        if (PL_markstack_ptr != temp) {
          /* truly void, because dXSARGS not invoked */
          PL_markstack_ptr = temp;
          XSRETURN_EMPTY; /* return empty stack */
        }
        /* must have used dXSARGS; list context implied */
        return; /* assume stack size is correct */

int
read_event (d)
	SV *	d

int
check_unix_password (checkpw)
	char *	checkpw

